@@ -20,5 +20,5 @@ except pkg_resources.DistributionNotFound:
     
 import os
 es_path = os.path.dirname(__file__)
-del os
+del os, pkg_resources
     
\ No newline at end of file
