@@ -14,13 +14,22 @@ for license details.
 
 import os
 bsm1_path = os.path.dirname(__file__)
+data_path = os.path.join(bsm1_path, 'data')
+results_path = os.path.join(bsm1_path, 'results')
+figures_path = os.path.join(bsm1_path, 'figures')
+del os
 
 from . import system
-
 from .system import *
 
+from . import model
+from .model import *
+
 __all__ = (
     'bsm1_path',
-    'system',
+    'data_path',
+    'results_path',
+    'figures_path',
     *system.__all__,
+    *model.__all__,
 	)
\ No newline at end of file
